/*
 * XMLTVImportPlugin.java
 *
 * This code is released to the public domain.
 *
 * Created by Mattias Larsson, ml@techno.org
 *
 * Created on April 21, 2003, 5:52 PM
 */
/* Modified by Lester Jacobs, April 28th, 2003
 * Code modified to extract the correct tuner channel
 * from the xmltv_single data.  Also reruns are now determined
 * from the <previously-shown> tag.
 * Search on: lester1
 */
/*Modified by Mike Brown 9/17/2003 to support multiple xmltv_single lineups
 * Up to 5 different xmltv_single lineups can be used.  
 * Just put each lineup in the SageTV directory with the names 1.xml, 2.xml, 3.xml, 4.xml and/or 5.xml.
 * You do not have to have all files files in the directory, but any that you chose in Sage, must be present.
 */
/* Modified by Mike Brown 9/18/2003
 * updated Show, Channel, and XMLTVPlugin to pass providerID 
 * in order to create unique ChanID's for duplicate tuner numbers amound providers
 */

package xmltv;

import java.io.FileInputStream;
import org.xml.sax.*;

import java.util.HashMap;
import java.util.TreeMap;
import java.util.Iterator;

/**
 *
 * More information at:
 *
 * http://mta.techno.org/epg/
 *
 * @author  mta
 */
public class XMLTVImportPlugin implements sage.EPGImportPlugin, ContentHandler, ErrorHandler {
    public String[][] prov = new String[][]{new String[]{ "1", "Lineup in 1.xml"}, {"2", "Lineup in 2.xml"},{"3", "Lineup in 3.xml"},{"4", "Lineup in 4.xml"},{"3", "Lineup in 5.xml"}};

    private sage.EPGDBPublic epg;

    // XML parsing scratch
    private Channel channel;
    private StringBuffer text;
    private HashMap channels;
    private Show show;
    private int thisProviderID;//mbrown

    /** Creates a new instance of XMLTVImportPlugin */
    public XMLTVImportPlugin() {
    }

    public String[][] getLocalMarkets() {
        return prov;
    }
    public String[][] getProviders(String zipCode) {
        return prov;
    }

    public boolean updateGuide(String providerID, sage.EPGDBPublic epg) {
        this.epg = epg;
        this.channels = new HashMap();

        try {
            XMLReader r = org.xml.sax.helpers.XMLReaderFactory.createXMLReader();
            
            try {
                r.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
            }catch (org.xml.sax.SAXNotRecognizedException e) {
                System.out.println("failed to set http://apache.org/xml/features/nonvalidating/load-external-dtd=false on "+r+"\n: "+e);
            }
            try {
                r.setFeature("http://xml.org/sax/features/nonvalidating/load-external-dtd", false);
            } catch (org.xml.sax.SAXNotRecognizedException e) {
                System.out.println("failed to set http://xml.org/sax/features/nonvalidating/load-external-dtd=false on "+r+"\n: "+e);
            }

            thisProviderID=Integer.parseInt(providerID);


            r.setEntityResolver(new EntityResolver() {
                public InputSource resolveEntity(String publicId, 
                        String systemId) throws SAXException, java.io.InvalidObjectException {
                    try {
                        System.out.println("Returning local XMLTV dtd");
                        return new InputSource(new FileInputStream("xmltv_single.dtd"));
                    } catch (Exception e){
                        return null;
                    }
                }
            });

            r.setContentHandler(this);
            r.setErrorHandler(this);

            try {
                InputSource i =new InputSource(new FileInputStream(providerID + ".xml"));
                i.setPublicId("xxx");
                i.setSystemId("file:xmltv_single.dtd");

                r.parse(i);
            }
            catch (java.io.FileNotFoundException e) {
                System.out.println("EPG data file not found.\n\n");
                this.epg.addChannelPublic("None", "No Data File", "None", 2147483647);
                TreeMap map = new TreeMap();
                String ch = "0";
                map.put(new Integer(2147483647),new String[]{ch});
                this.epg.setLineup(Long.parseLong(providerID),  map);
                return false;
            }
            catch (Exception e) {
                e.printStackTrace(System.out);
                return false;
            }

            TreeMap map = new TreeMap();
            int tunerchan = 2;
            for (Iterator i = this.channels.values().iterator(); i.hasNext(); ) {
                Channel c = (Channel)i.next();
                String ch;

                if (c.getTunerChannel() < 0) { // no real tuner channel available
                    ch = ""+tunerchan;
                    tunerchan++;
                } else { // real tuner available
                    ch = ""+c.getTunerChannel();
                }
                map.put(new Integer(c.getChannelId(thisProviderID)), new String[]{ ch });
            }

            this.epg.setLineup(Long.parseLong(providerID),  map);
            // provider hardcoded since sagetv doesnt call get providers

        } catch (Exception e) {
            e.printStackTrace(System.out);
        }



        return true;
    }

    /* **** XML Parsing **** */
    public void characters(char[] ch, int start, int length) throws SAXException {
        if (this.text != null) {
            this.text.append(ch, start, length);
        }
    }

    public void endDocument() throws SAXException {
    }

    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        String tag = qName.toLowerCase();
        boolean isShow = (this.show != null && this.text != null);
        try {
            if (tag.equals("channel")) {
                // this.channels.put(channel.id, channel); // lester1.o
                // System.out.println("channel: "+this.channel.longname); // lester1.o
                this.channel.EPGInsert(this.epg, thisProviderID);
                this.channel = null;
            } else if (tag.equals("programme")) {
                // add show here...
                this.show.EPGInsert(this.epg);
                this.show = null;
            } else if (tag.equals("sub-title") && isShow) {
                this.show.setEpisodeName(this.text.toString());
            } else if (tag.equals("desc") && isShow) {
                this.show.setDesc(this.text.toString());
            } else if (tag.equals("title") && isShow) {
                this.show.setTitle(this.text.toString());
            } else if (tag.equals("category") && isShow) {
                this.show.setCategory(this.text.toString());
            } else if (tag.equals("previously-shown") && isShow) { // lester1.sn
                this.show.setRerun(true);  // lester1.en
            } else if (tag.equals("display-name") && this.channel != null) {
                // System.out.println("setting display name to '"+this.text+"'\n");  // CKIM
                if (this.channel.isChannelIdSet()) {
                    // System.out.println("set was skipped. current channelId='"+(this.channel.getChannelId())+"'\n"); // CKIM
                } else {
                    this.channel.setDisplayName(this.text.toString(), thisProviderID);
                    // System.out.println("set was successful\n");  // CKIM
                }
            }
            this.text = null;
        } catch (Exception e) {
            if ( this.text != null )
                System.out.println("Error parsing end tag "+tag+" text=" +this.text.toString());
            else
                System.out.println("Error parsing end tag "+tag+" text=null" );		
            e.printStackTrace(System.out);
        }
    }

    public void endPrefixMapping(String prefix) throws SAXException {
    }

    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
    }

    public void setDocumentLocator(Locator locator) {
    }

    public void skippedEntity(String name) throws SAXException {
    }

    public void startDocument() throws SAXException {
    }

    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        String tag = qName.toLowerCase();
        try {
            if (tag.equals("channel")) {
                this.channel = new Channel();
                this.channel.setId(atts.getValue("id"));
                this.channels.put(atts.getValue("id"), channel); // lester1
            } else if (tag.equals("programme")) {
                this.show = new Show();
                // System.out.println("start: "+atts.getValue("start"));
                this.show.setStart(atts.getValue("start"));
                this.show.setEnd(atts.getValue("stop"));
                //  System.out.println("chan: "+atts.getValue("channel"));
                String channelAttr = atts.getValue("channel");
                if (channelAttr != null) {
                    Channel chan = (Channel)this.channels.get(channelAttr);
                    if (chan != null) {
                        this.show.setChannel(chan, thisProviderID);
                    } else { // never seen before channel, create it
                        chan = new Channel();
                        chan.setId(channelAttr);
                        this.channels.put(channelAttr, chan);
                        this.show.setChannel(chan, thisProviderID);
                        chan.EPGInsert(this.epg, thisProviderID);
                    }
                }

            } else if ((tag.equals("category") || tag.equals("desc") || tag.equals("title") ||
                    tag.equals("sub-title") || tag.equals("previously-shown")
            )
            && this.show != null) {
                this.text = new StringBuffer();
            } else if (tag.equals("display-name")) {
                this.text = new StringBuffer();
            }
        } catch (Exception e) {
            System.out.println("Error parsing start tag "+tag );		
            e.printStackTrace(System.out);
        }
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
    }

    public void processingInstruction(String target, String data) throws SAXException {
    }

    /** Receive notification of a recoverable error.
     *
     * <p>This corresponds to the definition of "error" in section 1.2
     * of the W3C XML 1.0 Recommendation.  For example, a validating
     * parser would use this callback to report the violation of a
     * validity constraint.  The default behaviour is to take no
     * action.</p>
     *
     * <p>The SAX parser must continue to provide normal parsing events
     * after invoking this method: it should still be possible for the
     * application to process the document through to the end.  If the
     * application cannot do so, then the parser should report a fatal
     * error even if the XML 1.0 recommendation does not require it to
     * do so.</p>
     *
     * <p>Filters may use this method to report other, non-XML errors
     * as well.</p>
     *
     * @param exception The error information encapsulated in a
     *                  SAX parse exception.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.SAXParseException
     *
     */
    public void error(SAXParseException exception) throws SAXException {
        System.out.println(exception.toString());
    }

    /** Receive notification of a non-recoverable error.
     *
     * <p>This corresponds to the definition of "fatal error" in
     * section 1.2 of the W3C XML 1.0 Recommendation.  For example, a
     * parser would use this callback to report the violation of a
     * well-formedness constraint.</p>
     *
     * <p>The application must assume that the document is unusable
     * after the parser has invoked this method, and should continue
     * (if at all) only for the sake of collecting addition error
     * messages: in fact, SAX parsers are free to stop reporting any
     * other events once this method has been invoked.</p>
     *
     * @param exception The error information encapsulated in a
     *                  SAX parse exception.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.SAXParseException
     *
     */
    public void fatalError(SAXParseException exception) throws SAXException {
        System.out.println(exception.toString());
    }

    /** Receive notification of a warning.
     *
     * <p>SAX parsers will use this method to report conditions that
     * are not errors or fatal errors as defined by the XML 1.0
     * recommendation.  The default behaviour is to take no action.</p>
     *
     * <p>The SAX parser must continue to provide normal parsing events
     * after invoking this method: it should still be possible for the
     * application to process the document through to the end.</p>
     *
     * <p>Filters may use this method to report other, non-XML warnings
     * as well.</p>
     *
     * @param exception The warning information encapsulated in a
     *                  SAX parse exception.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.SAXParseException
     *
     */
    public void warning(SAXParseException exception) throws SAXException {
        System.out.println(exception.toString());
    }

}
